package com.hywa.banktest.bankframework.socketHandle;

import com.alibaba.fastjson.JSONObject;
import com.hywa.banktest.bankframework.pack.Dlmaps;
import com.hywa.banktest.bankframework.schedule.BankRequsetProcess;
import com.hywa.banktest.bankframework.utils.XmlUtils;
import com.hywa.banktest.kit.SpringContext;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import java.io.*;
import java.net.InetSocketAddress;
import java.nio.ByteBuffer;
import java.nio.channels.SelectionKey;
import java.nio.channels.Selector;
import java.nio.channels.ServerSocketChannel;
import java.nio.channels.SocketChannel;
import java.nio.charset.Charset;
import java.util.Iterator;

@Slf4j
@Component
public class SocketService {

    @Value("${local.post.socket.port}")
    private int localPort;

    /**
     * 开启socket服务，永久监听
     * 接收物理设备上报的数据
     *
     * @param port
     * @return void
     */
    public void createSocketService(int port) {
        Selector selector = null;
        ServerSocketChannel serverChannel = null;
        try {
            //获取一个ServerSocket通道
            serverChannel = ServerSocketChannel.open();
            serverChannel.configureBlocking(false);
            serverChannel.socket().bind(new InetSocketAddress(port));
            //获取通道管理器
            selector = Selector.open();
            //将通道管理器与通道绑定，并为该通道注册SelectionKey.OP_ACCEPT事件，
            //只有当该事件到达时，Selector.select()会返回，否则一直阻塞。
            serverChannel.register(selector, SelectionKey.OP_ACCEPT);

            while (selector.select() > 0) {
                //选择注册过的io操作的事件
                Iterator<SelectionKey> it = selector.selectedKeys().iterator();
                while (it.hasNext()) {
                    SelectionKey readyKey = it.next();
                    //删除已选key，防止重复处理
                    it.remove();
                    if (readyKey.isAcceptable()) {
                        ServerSocketChannel serverSocketChannel = (ServerSocketChannel) readyKey.channel();
                        SocketChannel socketChannel = serverSocketChannel.accept();

                        socketChannel.configureBlocking(false);
                        // 连接成功后，注册接收服务器消息的事件
                        socketChannel.register(selector, SelectionKey.OP_READ | SelectionKey.OP_WRITE);
                    } else if (readyKey.isValid() && readyKey.isReadable()) {
                        SocketChannel socketChannel = (SocketChannel) readyKey.channel();
                        Object obj = receiveData(socketChannel, readyKey);
                        String resStr = "";
                        if (obj != null) {
                            try {
                                log.info("收到了请求数据"+ JSONObject.toJSONString(obj));
                                StringBuffer result = new StringBuffer(obj.toString());
                                log.info("服务端收到报文：" + result);
                                result.delete(0, result.indexOf("<"));
                                if (result.toString().contains("***"))
                                    result.delete(result.indexOf("***"), result.length());

                                log.info("服务端处理xml报文：" + result);
                                Dlmaps dlmaps = XmlUtils.fromXML(result.toString(), Dlmaps.class);
                                BankRequsetProcess bankRequsetProcess = (BankRequsetProcess) SpringContext.getBean("bankRequsetProcess");
                                Dlmaps resultDlmaps = bankRequsetProcess.process(dlmaps);
                                if ("8900".equals(dlmaps.getPUB().getTRADE_ID() )){
                                    resultDlmaps.setIN(null);
                                }

                                resStr = dlmaps2RequestStr(resultDlmaps);

                            } catch (Exception e) {

                            }

                            sendData(socketChannel, resStr);
                        }
                    }
                }
            }
        } catch (Exception e) {
            log.error("ServerSocket通道异常：" + e);
            e.printStackTrace();
        } finally {
            try {
                selector.close();
                if (serverChannel != null) {
                    serverChannel.close();
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }


    private String dlmaps2RequestStr(Dlmaps requestPacket) throws UnsupportedEncodingException {
        String xmlRequest = XmlUtils.toXML(requestPacket);
        StringBuffer requestPac = new StringBuffer();
        requestPac.append("TEB").append(requestPacket.getPUB().getTRADE_ID()).append(requestPacket.getPUB().getDPT_ID()).append(xmlRequest);

        StringBuffer requestStr = new StringBuffer(String.format("%04d", requestPac.toString().getBytes("UTF-8").length + 3)).append(requestPac).append("***");
        return requestStr.toString();
    }

    /**
     * 接收socket数据
     *
     * @param socketChannel
     * @param readyKey
     * @return Object
     */
    private static Object receiveData(SocketChannel socketChannel, SelectionKey readyKey) {
        Object obj = null;
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        ByteBuffer intBuffer = ByteBuffer.allocate(4);
        ByteBuffer objBuffer = ByteBuffer.allocate(1024);
        int size = 0;
        int sum = 0;
        int objlen = 0;
        byte[] bytes = null;
        try {
            ByteBuffer buffer = ByteBuffer.allocate(1024);
            buffer.clear();
            // 读取信息获得读取的字节数
            long bytesRead = socketChannel.read(buffer);
            if (bytesRead == -1) {
                // 没有读取到内容的情况
                socketChannel.close();
            } else {
                buffer.flip();
                obj = Charset.forName("UTF-8").newDecoder().decode(buffer).toString().trim();
            }
            readyKey.interestOps(SelectionKey.OP_READ | SelectionKey.OP_WRITE);
        } catch (Exception e) {
            log.error("远程主机强迫关闭了一个现有的连接:" + readyKey.channel());
            //e.printStackTrace();
            readyKey.cancel();
            try {
                socketChannel.socket().close();
                socketChannel.close();
            } catch (IOException e1) {
                e1.printStackTrace();
            }
            //e.printStackTrace();
        } finally {
            try {
                baos.close();
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        return obj;
    }

    /**
     * 响应socket数据
     *
     * @param socketChannel
     * @param obj
     * @return void
     */
    private static void sendData(SocketChannel socketChannel, Object obj) {
        // 准备发送的文本
        String sendString = obj.toString();
        ByteBuffer buffer = null;
        try {
            buffer = ByteBuffer.wrap(sendString.getBytes("gbk"));
            socketChannel.write(buffer);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * byte数组中取int数值
     *
     * @param ary    byte数组
     * @param offset 从数组的第offset位开始
     * @return int数值
     */
    public static int bytesToInt(byte[] ary, int offset) {
        int value;
        value = (int) ((ary[offset] & 0xFF)
                | ((ary[offset + 1] << 8) & 0xFF00)
                | ((ary[offset + 2] << 16) & 0xFF0000)
                | ((ary[offset + 3] << 24) & 0xFF000000));
        return value;
    }

    public static Object ByteToObject(byte[] bytes) {
        Object obj = null;
        try {
            // bytearray to object
            ByteArrayInputStream bi = new ByteArrayInputStream(bytes);
            ObjectInputStream oi = new ObjectInputStream(bi);
            obj = oi.readObject();
            bi.close();
            oi.close();
        } catch (Exception e) {
            //e.printStackTrace();
        }
        return obj;
    }

    public static byte[] ObjectToByte(Object obj) {
        byte[] bytes = null;
        try {
            // object to bytearray
            ByteArrayOutputStream bo = new ByteArrayOutputStream();
            ObjectOutputStream oo = new ObjectOutputStream(bo);
            oo.writeObject(obj);
            bytes = bo.toByteArray();
            bo.close();
            oo.close();
        } catch (Exception e) {
            e.printStackTrace();
        }
        return bytes;
    }

    /**
     * 通过线程启动socket服务
     */
    class InitSocket implements Runnable {
        @Override
        public void run() {
            try {
                createSocketService(localPort);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }

    @PostConstruct
    public void init() {
        log.info("启动whxph的socket服务开始：" + System.currentTimeMillis());
        new Thread(new InitSocket()).start();
        log.info("启动whxph的socket服务成功：" + System.currentTimeMillis());
    }


}
